@@ -23,12 +23,11 @@ _____       _    _    Corso   Italia,  178
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 static char rcsid[] = "$Id: catalog.cpp 4.0 1998/10/27 16:19:54 picoSoft Exp $";
-# include <sql.h>
-# include <sqlext.h>
 # ifdef WIN32
 # include <windows.h>
 # endif
-# include <stdio.h>
+# include <sql.h>
+# include <sqlext.h>
 # include "sqlhndls.h"
 # include "dberror.h"
 extern "C" {
@@ -74,12 +73,7 @@ RETCODE SQL_API SQLColumns(
    argv[9] = &cbColumnName;
    
    Exception *ex = Exception_new();
-   if (!Invoke_invoke (myHstmt->hdbc->call,ex,"SQLColumns",argv)) {
-      char stat[6];
-      sprintf (stat,"08%03d", ex->code);
-      AddError (0, 0, myHstmt, stat, ex->code, ex->description);
-      Exception_log(ex);
-   }
+   doCall (myHstmt,ex,"SQLColumns",argv);
    if (argv[2] != szTableQualifier)
       delete (char *) argv[2];
    if (argv[4] != szTableOwner)
@@ -133,12 +127,7 @@ RETCODE SQL_API SQLTables(
    argv[9] = &cbTableType;
    
    Exception *ex = Exception_new();
-   if (!Invoke_invoke (myHstmt->hdbc->call,ex,"SQLTables",argv)) {
-      char stat[6];
-      sprintf (stat,"08%03d", ex->code);
-      AddError (0, 0, myHstmt, stat, ex->code, ex->description);
-      Exception_log(ex);
-   }
+   doCall (myHstmt,ex,"SQLTables",argv);
    if (argv[2] != szTableQualifier)
       delete (char *) argv[2];
    if (argv[4] != szTableOwner)
@@ -197,12 +186,7 @@ RETCODE SQL_API SQLStatistics(
       argv[9] = &fAccuracy;
 
       Exception *ex = Exception_new();
-      if (!Invoke_invoke (myHstmt->hdbc->call,ex,"SQLStatistics",argv)) {
-         char stat[6];
-         sprintf (stat,"08%03d", ex->code);
-         AddError (0, 0, myHstmt, stat, ex->code, ex->description);
-         Exception_log(ex);
-      }
+      doCall (myHstmt,ex,"SQLStatistics",argv);
       if (argv[2] != szTableQualifier)
          delete (char *) argv[2];
       if (argv[4] != szTableOwner)
@@ -255,12 +239,7 @@ RETCODE SQL_API SQLSpecialColumns(
       argv[10] = &fNullable;
 
       Exception *ex = Exception_new();
-      if (!Invoke_invoke (myHstmt->hdbc->call,ex,"SQLSpecialColumns",argv)) {
-         char stat[6];
-         sprintf (stat,"08%03d", ex->code);
-         AddError (0, 0, myHstmt, stat, ex->code, ex->description);
-         Exception_log(ex);
-      }
+      doCall (myHstmt,ex,"SQLSpecialColumns",argv);
       if (argv[3] != szTableQualifier)
          delete (char *) argv[3];
       if (argv[5] != szTableOwner)
@@ -307,12 +286,7 @@ RETCODE SQL_API SQLTablePrivileges(
       argv[7] = &cbTableName;
 
       Exception *ex = Exception_new();
-      if (!Invoke_invoke (myHstmt->hdbc->call,ex,"SQLTablePrivileges",argv)) {
-         char stat[6];
-         sprintf (stat,"08%03d", ex->code);
-         AddError (0, 0, myHstmt, stat, ex->code, ex->description);
-         Exception_log(ex);
-      }
+      doCall (myHstmt,ex,"SQLTablePrivileges",argv);
       if (argv[2] != szTableQualifier)
          delete (char *) argv[2];
       if (argv[4] != szTableOwner)
@@ -364,12 +338,7 @@ RETCODE SQL_API SQLColumnPrivileges(
       argv[9] = &cbColumnName;
 
       Exception *ex = Exception_new();
-      if (!Invoke_invoke (myHstmt->hdbc->call,ex,"SQLColumnPrivileges",argv)) {
-         char stat[6];
-         sprintf (stat,"08%03d", ex->code);
-         AddError (0, 0, myHstmt, stat, ex->code, ex->description);
-         Exception_log(ex);
-      }
+      doCall (myHstmt,ex,"SQLColumnPrivileges",argv);
       if (argv[2] != szTableQualifier)
          delete (char *) argv[2];
       if (argv[4] != szTableOwner)
@@ -418,12 +387,7 @@ RETCODE SQL_API SQLPrimaryKeys(
       argv[7] = &cbTableName;
 
       Exception *ex = Exception_new();
-      if (!Invoke_invoke (myHstmt->hdbc->call,ex,"SQLPrimaryKeys",argv)) {
-         char stat[6];
-         sprintf (stat,"08%03d", ex->code);
-         AddError (0, 0, myHstmt, stat, ex->code, ex->description);
-         Exception_log(ex);
-      }
+      doCall (myHstmt,ex,"SQLPrimaryKeys",argv);
       if (argv[2] != szTableQualifier)
          delete (char *) argv[2];
       if (argv[4] != szTableOwner)
@@ -485,12 +449,7 @@ RETCODE SQL_API SQLForeignKeys(
       argv[13] = &cbFkTableName;
 
       Exception *ex = Exception_new();
-      if (!Invoke_invoke (myHstmt->hdbc->call,ex,"SQLForeignKeys",argv)) {
-         char stat[6];
-         sprintf (stat,"08%03d", ex->code);
-         AddError (0, 0, myHstmt, stat, ex->code, ex->description);
-         Exception_log(ex);
-      }
+      doCall (myHstmt,ex,"SQLForeignKeys",argv);
       if (argv[2] != szPkTableQualifier)
          delete (char *) argv[2];
       if (argv[4] != szPkTableOwner)
@@ -543,12 +502,7 @@ RETCODE SQL_API SQLProcedures(
       argv[7] = &cbProcName;
 
       Exception *ex = Exception_new();
-      if (!Invoke_invoke (myHstmt->hdbc->call,ex,"SQLProcedures",argv)) {
-         char stat[6];
-         sprintf (stat,"08%03d", ex->code);
-         AddError (0, 0, myHstmt, stat, ex->code, ex->description);
-         Exception_log(ex);
-      }
+      doCall (myHstmt,ex,"SQLProcedures",argv);
       if (argv[2] != szProcQualifier)
          delete (char *) argv[2];
       if (argv[4] != szProcOwner)
@@ -600,12 +554,7 @@ RETCODE SQL_API SQLProcedureColumns(
       argv[9] = &cbColumnName;
 
       Exception *ex = Exception_new();
-      if (!Invoke_invoke (myHstmt->hdbc->call,ex,"SQLProcedureColumns",argv)) {
-         char stat[6];
-         sprintf (stat,"08%03d", ex->code);
-         AddError (0, 0, myHstmt, stat, ex->code, ex->description);
-         Exception_log(ex);
-      }
+      doCall (myHstmt,ex,"SQLProcedureColumns",argv);
       if (argv[2] != szProcQualifier)
          delete (char *) argv[2];
       if (argv[4] != szProcOwner)
