@@ -638,8 +638,8 @@ RETCODE SQL_API SQLDisconnect(
    argv[0] = &Return;
    argv[1] = &myHdbc->srvHdbc;
    Exception *ex = Exception_new();
-/* Ci sarebbe anche da interrompere la connessione !! */
-   doCall (myHdbc, ex, "SQLDisconnect", argv);
+   if (doCall (myHdbc, ex, "SQLDisconnect", argv) && Return == SQL_SUCCESS)
+      doCall (myHdbc, ex, "SQLFreeConnect", argv);
    Exception_delete (ex);
    return (RETCODE) Return;
 }
@@ -648,18 +648,11 @@ static RETCODE mySQLFreeConnect(
    HDBC   hdbc)
 {
    ClntHdbc *myHdbc = (ClntHdbc *) hdbc;
-   CORBA_Long Return = SQL_ERROR;
+   CORBA_Long Return = SQL_SUCCESS;
 
-   void *argv[2];
-   argv[0] = &Return;
-   argv[1] = &myHdbc->srvHdbc;
-   Exception *ex = Exception_new();
-   if (doCall (myHdbc, ex, "SQLFreeConnect", argv) && Return == SQL_SUCCESS) {
-      IiopObj_delete (myHdbc->obj);
-      Invoke_delete (myHdbc->call);
-      delete myHdbc;
-   }
-   Exception_delete (ex);
+   IiopObj_delete (myHdbc->obj);
+   Invoke_delete (myHdbc->call);
+   delete myHdbc;
    return (RETCODE) Return;
 }
 
