@@ -24,8 +24,6 @@ _____       _    _    Corso   Italia,  178
 */
 static char rcsid[] = "$Id$";
 
-# include <sql.h>
-# include <sqlext.h>
 # ifdef WIN32
 # include <windows.h>
 static char User[9 + 1];
@@ -35,13 +33,15 @@ extern HINSTANCE ghInstance;
 # include <stdlib.h>
 # include <string.h>
 # endif
+# include <sql.h>
+# include <sqlext.h>
 # include <stdio.h>
 # include "sqlhndls.h"
 # include "dberror.h"
 # include "envirini.h"
 
 # define MAXDBDIR 128
-# define DRIVER_VER "1.00.0399"
+# define DRIVER_VER "1.06.0000"
 
 # define DRIVER_NAME "PICOSQLN.DLL"
 # define SERVER_NAME "PICOSQLN" 
@@ -188,15 +188,7 @@ RETCODE SQL_API SQLGetInfo(
        NUM(SWORD,SQL_CB_PRESERVE);
        break;
     case SQL_DATA_SOURCE_READ_ONLY:
-# ifdef ACU
-       if (IsDriverReadOnly()) {
-          STR("Y");
-       } else {
-          STR("N");
-       }
-# else
        STR("N");
-# endif       
        break;
     case SQL_DEFAULT_TXN_ISOLATION:
        NUM(SDWORD,SQL_TXN_READ_UNCOMMITTED);
@@ -543,16 +535,217 @@ RETCODE SQL_API SQLGetTypeInfo(
    argv[1] = (void *) &myHstmt->srvHstmt;
    argv[2] = (void *) &fSqlType;
    Exception *ex = Exception_new();
-   if (!Invoke_invoke (myHstmt->hdbc->call, ex, "SQLGetTypeInfo", argv)) {
-      char stat[6];
-      sprintf (stat,"08%03d", ex->code);
-      AddError (0, 0, myHstmt, stat, ex->code, ex->description);
-      Exception_log(ex);
-   }
+   doCall (myHstmt, ex, "SQLGetTypeInfo", argv);
    Exception_delete (ex);
    return (RETCODE) Return;
 }
 
+static inline int
+getFunction (unsigned short id)
+{
+   switch (id) {
+   case SQL_API_SQLALLOCCONNECT:
+      return SQL_TRUE;
+   case SQL_API_SQLALLOCENV:
+      return SQL_TRUE;
+   case SQL_API_SQLALLOCSTMT:
+      return SQL_TRUE;
+   case SQL_API_SQLBINDCOL:
+      return SQL_TRUE;
+   case SQL_API_SQLCANCEL:
+      return SQL_TRUE;
+   case SQL_API_SQLCOLATTRIBUTES:
+      return SQL_TRUE;
+/*
+   case SQL_API_SQLCOLATTRIBUTE:
+      return SQL_TRUE;
+*/
+   case SQL_API_SQLCONNECT:
+      return SQL_TRUE;
+   case SQL_API_SQLDESCRIBECOL:
+      return SQL_TRUE;
+   case SQL_API_SQLDISCONNECT:
+      return SQL_TRUE;
+   case SQL_API_SQLERROR:
+      return SQL_TRUE;
+   case SQL_API_SQLEXECDIRECT:
+      return SQL_TRUE;
+   case SQL_API_SQLEXECUTE:
+      return SQL_TRUE;
+   case SQL_API_SQLFETCH:
+      return SQL_TRUE;
+   case SQL_API_SQLFREECONNECT:
+      return SQL_TRUE;
+   case SQL_API_SQLFREEENV:
+      return SQL_TRUE;
+   case SQL_API_SQLFREESTMT:
+      return SQL_TRUE;
+   case SQL_API_SQLGETCURSORNAME:
+      return SQL_TRUE;
+   case SQL_API_SQLNUMRESULTCOLS:
+      return SQL_TRUE;
+   case SQL_API_SQLPREPARE:
+      return SQL_TRUE;
+   case SQL_API_SQLROWCOUNT:
+      return SQL_TRUE;
+   case SQL_API_SQLSETCURSORNAME:
+      return SQL_TRUE;
+   case SQL_API_SQLSETPARAM:
+      return SQL_TRUE;
+   case SQL_API_SQLTRANSACT:
+      return SQL_TRUE;
+   case SQL_API_SQLCOLUMNS:
+      return SQL_TRUE;
+   case SQL_API_SQLDRIVERCONNECT:
+      return SQL_TRUE;
+   case SQL_API_SQLGETCONNECTOPTION:
+      return SQL_TRUE;
+   case SQL_API_SQLGETDATA:
+      return SQL_TRUE;
+   case SQL_API_SQLGETFUNCTIONS:
+      return SQL_TRUE;
+   case SQL_API_SQLGETINFO:
+      return SQL_TRUE;
+   case SQL_API_SQLGETSTMTOPTION:
+      return SQL_TRUE;
+   case SQL_API_SQLGETTYPEINFO:
+      return SQL_TRUE;
+   case SQL_API_SQLPARAMDATA:
+      return SQL_TRUE;
+   case SQL_API_SQLPUTDATA:
+      return SQL_TRUE;
+   case SQL_API_SQLSETCONNECTOPTION:
+      return SQL_TRUE;
+   case SQL_API_SQLSETSTMTOPTION:
+      return SQL_TRUE;
+   case SQL_API_SQLSPECIALCOLUMNS:
+      return SQL_TRUE;
+   case SQL_API_SQLSTATISTICS:
+      return SQL_TRUE;
+   case SQL_API_SQLTABLES:
+      return SQL_TRUE;
+   case SQL_API_SQLBROWSECONNECT:
+      return SQL_FALSE;
+   case SQL_API_SQLCOLUMNPRIVILEGES:
+      return SQL_TRUE;
+   case SQL_API_SQLDATASOURCES:
+      return SQL_TRUE;
+   case SQL_API_SQLDESCRIBEPARAM:
+      return SQL_TRUE;
+   case SQL_API_SQLEXTENDEDFETCH:
+      return SQL_TRUE;
+   case SQL_API_SQLFOREIGNKEYS:
+      return SQL_TRUE;
+   case SQL_API_SQLMORERESULTS:
+      return SQL_TRUE;
+   case SQL_API_SQLNATIVESQL:
+      return SQL_TRUE;
+   case SQL_API_SQLNUMPARAMS:
+      return SQL_TRUE;
+   case SQL_API_SQLPARAMOPTIONS:
+      return SQL_FALSE;
+   case SQL_API_SQLPRIMARYKEYS:
+      return SQL_TRUE;
+   case SQL_API_SQLPROCEDURECOLUMNS:
+      return SQL_TRUE;
+   case SQL_API_SQLPROCEDURES:
+      return SQL_TRUE;
+   case SQL_API_SQLSETPOS:
+      return SQL_TRUE;
+   case SQL_API_SQLSETSCROLLOPTIONS:
+      return SQL_FALSE;
+   case SQL_API_SQLTABLEPRIVILEGES:
+      return SQL_TRUE;
+   case SQL_API_SQLDRIVERS:
+      return SQL_TRUE;
+   case SQL_API_SQLBINDPARAMETER:
+      return SQL_TRUE;
+   case SQL_API_SQLALLOCHANDLE:
+      return SQL_TRUE;
+   case SQL_API_SQLBINDPARAM:
+      return SQL_TRUE;
+   case SQL_API_SQLCLOSECURSOR:
+      return SQL_TRUE;
+   case SQL_API_SQLCOPYDESC:
+      return SQL_FALSE;
+   case SQL_API_SQLENDTRAN:
+      return SQL_TRUE;
+   case SQL_API_SQLFETCHSCROLL:
+      return SQL_TRUE;
+   case SQL_API_SQLFREEHANDLE:
+      return SQL_TRUE;
+   case SQL_API_SQLGETCONNECTATTR:
+      return SQL_TRUE;
+   case SQL_API_SQLGETDESCFIELD:
+      return SQL_FALSE;
+   case SQL_API_SQLGETDESCREC:
+      return SQL_FALSE;
+   case SQL_API_SQLGETDIAGFIELD:
+      return SQL_TRUE;
+   case SQL_API_SQLGETDIAGREC:
+      return SQL_TRUE;
+   case SQL_API_SQLGETENVATTR:
+      return SQL_TRUE;
+   case SQL_API_SQLGETSTMTATTR:
+      return SQL_TRUE;
+   case SQL_API_SQLSETCONNECTATTR:
+      return SQL_TRUE;
+   case SQL_API_SQLSETDESCFIELD:
+      return SQL_FALSE;
+   case SQL_API_SQLSETDESCREC:
+      return SQL_FALSE;
+   case SQL_API_SQLSETENVATTR:
+      return SQL_TRUE;
+   case SQL_API_SQLSETSTMTATTR:
+      return SQL_TRUE;
+   case SQL_API_SQLBULKOPERATIONS:
+      return SQL_FALSE;
+   default:
+      break;
+   }
+   return SQL_FALSE;
+}
+
+RETCODE SQL_API
+SQLGetFunctions(HDBC hdbc,unsigned short fFunction,
+                          unsigned short * pfExist)
+{
+   RETCODE Return = SQL_SUCCESS;
+   register int i, j;
+
+   switch (fFunction) {
+   case SQL_API_ALL_FUNCTIONS:
+      for (i = 0; i < 100; i++)
+         pfExist[i] = getFunction (i);
+      break;
+   case SQL_API_ODBC3_ALL_FUNCTIONS:
+      for (i = 0; i < SQL_API_ODBC3_ALL_FUNCTIONS_SIZE; i++) {
+         j = i << 4;
+         pfExist[i] = ((getFunction (j++) & 0x1) << 15) &
+                      ((getFunction (j++) & 0x1) << 14) &
+                      ((getFunction (j++) & 0x1) << 13) &
+                      ((getFunction (j++) & 0x1) << 12) &
+                      ((getFunction (j++) & 0x1) << 11) &
+                      ((getFunction (j++) & 0x1) << 10) &
+                      ((getFunction (j++) & 0x1) << 9) &
+                      ((getFunction (j++) & 0x1) << 8) &
+                      ((getFunction (j++) & 0x1) << 7) &
+                      ((getFunction (j++) & 0x1) << 6) &
+                      ((getFunction (j++) & 0x1) << 5) &
+                      ((getFunction (j++) & 0x1) << 4) &
+                      ((getFunction (j++) & 0x1) << 3) &
+                      ((getFunction (j++) & 0x1) << 2) &
+                      ((getFunction (j++) & 0x1) << 1) &
+                       (getFunction (j++) & 0x1);
+      }
+      break;
+   default:
+      *pfExist = getFunction (fFunction);
+      break;
+   }
+   return Return;
+}
+
 RETCODE SQL_API Dummy()
 {
    return SQL_SUCCESS;
