@@ -210,13 +210,15 @@ Connection_connect (Connection *this, Exception *ex)
          memcpy (&this->sin.sin_addr, this->hp->h_addr, this->hp->h_length);
          this->sin.sin_port = htons(this->obj->port);
          this->sin.sin_family = this->hp->h_addrtype;
+         this->type = Connection_SOCKET;
          if (connect(this->sock,
                      (struct sockaddr *)&this->sin,
                       sizeof(this->sin))==0) {
-            this->type = Connection_SOCKET;
             Return = CORBA_True;
-         } else
+         } else {
             SocketError("connect", ex);
+            Connection_close (this);
+         }
       }
    return Return;
 }
