@@ -44,6 +44,22 @@ checkEndian ()
       return CORBA_True; 
 }
 
+static void
+swapDouble (CORBA_Double *d)
+{
+   CORBA_ULong sw;
+   union uu {
+      CORBA_Double dbl;
+      struct ss {
+         CORBA_ULong w1;
+         CORBA_ULong w2;
+      } s;
+   } *ud = (union uu*) d;
+   sw = ud->s.w1;
+   ud->s.w1 = ud->s.w2;
+   ud->s.w2 = sw;
+}
+
 Message *
 Message_new ()
 {
@@ -113,15 +129,15 @@ Message_getBuffer (Message *This, unsigned int reqLen, CORBA_Boolean byteOrd)
    return This->logicBuffer;
 }
 
-CORBA_Octet *
+unsigned long
 Message_skipBytes (Message *This, unsigned int nBytes)
 {
-   CORBA_Octet *Return;
+   unsigned long Return;
    if (This->unusedSpace < nBytes &&
        Message_grow (This, nBytes - This->unusedSpace) == CORBA_False)
-      Return = (CORBA_Octet *) 0;
+      Return = 0;
    else {
-      Return = This->currPnt;
+      Return = This->currPnt - This->logicBuffer;
       This->currPnt += nBytes;
       This->unusedSpace -= nBytes;
    }
@@ -201,6 +217,9 @@ Message_putDouble (Message *This, CORBA_Double c)
       return CORBA_False;
    This->currPnt += align;
    *((CORBA_Double *)This->currPnt) = c;
+# ifdef ARM_LE
+   swapDouble ((CORBA_Double *)This->currPnt);
+# endif
    This->unusedSpace -= align + sizeof(c);
    This->currPnt += sizeof(c);
    return CORBA_True;
@@ -216,9 +235,9 @@ Message_putString (Message *This, CORBA_String c)
    } else {
       int size = strlen (c) + 1;
       if (Message_putLong(This, size)) {
-         char *str = Message_skipBytes (This, size);
-         if (str) {
-            memcpy (str, c, size);
+         unsigned long pnt = Message_skipBytes (This, size);
+         if (pnt) {
+            memcpy (This->logicBuffer + pnt, c, size);
             Return = CORBA_True;
          }
       }
@@ -234,9 +253,9 @@ Message_putPrincipal (Message *This, CORBA_Principal *c)
    if (c == 0) {
       Return = Message_putULong (This, 0);
    } else if ((Return = Message_putLong(This, c->size))) {
-      unsigned char *pnt = Message_skipBytes (This, c->size);
+      unsigned long pnt = Message_skipBytes (This, c->size);
       if (pnt)
-         memcpy (pnt, c->data, c->size);
+         memcpy (This->logicBuffer + pnt, c->data, c->size);
       else
          Return = CORBA_False;
    } else
@@ -278,7 +297,7 @@ Message_putSequence (Message *This, Exception *ex, const CORBA_TCKindDesc *k,
    if (k->suppl) {
       CORBA_Sequence *seq = (CORBA_Sequence *) data;
       unsigned int i;
-      char *pnt;
+      unsigned long pnt;
       int size;
       Return = Message_putULong (This, seq->count);
       switch (k->suppl[0]->kind) {
@@ -295,13 +314,14 @@ Message_putSequence (Message *This, Exception *ex, const CORBA_TCKindDesc *k,
          if (size == 1) {
             pnt = Message_skipBytes (This, seq->count);
             if (pnt)
-               memcpy (pnt, data, seq->count);
+               memcpy (This->logicBuffer + pnt, data, seq->count);
             else
                Return = CORBA_False;
          } else {
-            for (pnt = (char *) seq->data, i = 0; i < seq->count && Return;
-                                                              i++, pnt += size)
-               Return = Message_encode (This, ex, k->suppl[0], pnt);
+            char *pp;
+            for (pp = (char *) seq->data, i = 0; i < seq->count && Return;
+                                                              i++, pp += size)
+               Return = Message_encode (This, ex, k->suppl[0], pp);
          }
          break;
       default:
@@ -390,12 +410,12 @@ Message_getString (Message *This, CORBA_String *c, CORBA_Boolean autoAlloc)
 
    if ((Return = Message_getULong(This, &len))) {
       if (len > 0) {
-         char *str = Message_skipBytes (This, len);
-         if (str) {
+         unsigned long pnt = Message_skipBytes (This, len);
+         if (pnt) {
             if (autoAlloc)
                *c = malloc (len);
             if (*c)
-               memcpy (*c, str, len);
+               memcpy (*c, This->logicBuffer + pnt, len);
          } else
             Return = CORBA_False;
       }
@@ -410,12 +430,12 @@ Message_getPrincipal(Message *This,CORBA_Principal **c,CORBA_Boolean autoAlloc)
    CORBA_ULong len;
 
    if ((Return = Message_getULong(This, &len))) {
-      char *pnt = Message_skipBytes (This, len);
+      unsigned long pnt = Message_skipBytes (This, len);
       if (pnt) {
          if (autoAlloc)
             *c = CORBA_Object_new (&TCD_Principal, len);
          if (*c && (*c)->data)
-            memcpy ((*c)->data, pnt, len);
+            memcpy ((*c)->data, This->logicBuffer + pnt, len);
       } else
          Return = CORBA_False;
    }
@@ -431,6 +451,9 @@ Message_getDouble (Message *This, CORBA_Double *c)
       return CORBA_False;
    This->currPnt += align;
    if (c) {
+# ifdef ARM_LE
+      swapDouble ((CORBA_Double *)This->currPnt);
+# endif
       if (This->msgByteOrder == This->myByteOrder) {
          *c = *((CORBA_Double *)This->currPnt);
       } else {
@@ -493,7 +516,7 @@ Message_getSequence(Message *This, Exception *ex, const CORBA_TCKindDesc *k,
    if (k->suppl && (Return = Message_getULong(This, &len))) {
       CORBA_Sequence *seq;
       unsigned int i;
-      char *pnt;
+      unsigned long pnt;
       int size;
       switch (k->suppl[0]->kind) {
       case tk_short:
@@ -508,23 +531,24 @@ Message_getSequence(Message *This, Exception *ex, const CORBA_TCKindDesc *k,
          size = Feature[k->suppl[0]->kind].size;
          if (autoAlloc) {
             seq=(CORBA_Sequence *)(*data = malloc (sizeof(CORBA_Sequence)));
-            seq->count = len;
             seq->data = malloc (size * seq->count);
          } else {
             seq = (CORBA_Sequence *) *data;
          }
+         seq->count = len;
          if (*data) {
             if (size == 1) {
                pnt = Message_skipBytes (This, seq->count);
                if (pnt)
-                  memcpy (seq->data, pnt, seq->count);
+                  memcpy (seq->data, This->logicBuffer + pnt, seq->count);
                else
                   Return = CORBA_False;
             } else {
-               for (pnt = (char *) seq->data, i = 0; i < seq->count && Return;
-                                                               i++, pnt += size)
+               char *pp;
+               for (pp = (char *) seq->data, i = 0; i < seq->count && Return;
+                                                               i++, pp += size)
                   Return = Message_decode (This, ex, k->suppl[0],
-                                                   (void**)&pnt, CORBA_False);
+                                                   (void**)&pp, CORBA_False);
             }
          }
          break;
@@ -552,7 +576,7 @@ Message_encode (Message *This, Exception *ex, const CORBA_TCKindDesc *k,
                 const void *data)
 {
    CORBA_Boolean Return = CORBA_True;
-   CORBA_Octet *ulpnt;
+   unsigned long ulpnt;
 
    switch (k->kind) {
    case tk_null:
@@ -659,8 +683,9 @@ Message_encode (Message *This, Exception *ex, const CORBA_TCKindDesc *k,
                } else {
                   Message_putULong (This, 0);
                }
-               *((CORBA_ULong *) ulpnt) = (char *)This->currPnt - 
-                                      (char *) (ulpnt + sizeof(CORBA_ULong));
+               *((CORBA_ULong *) (This->logicBuffer + ulpnt)) =
+                   (char *)This->currPnt -
+                   (char *)(This->logicBuffer + ulpnt + sizeof(CORBA_ULong));
                break;
             case tk_sequence:
                ulpnt = Message_skipBytes (This, sizeof (CORBA_ULong));
@@ -674,8 +699,9 @@ Message_encode (Message *This, Exception *ex, const CORBA_TCKindDesc *k,
                   Return = Message_encode(This,ex, &TCD_TypeCode, &TCD_null);
                if (Return) {
                   Return = Message_putULong (This, 0);
-                  *((CORBA_ULong *) ulpnt) = (char *)This->currPnt - 
-                                         (char *) (ulpnt + sizeof(CORBA_ULong));
+                  *((CORBA_ULong *) (This->logicBuffer + ulpnt)) =
+                      (char *)This->currPnt - 
+                      (char *)(This->logicBuffer + ulpnt + sizeof(CORBA_ULong));
                }
                break;
             case tk_array:
@@ -691,8 +717,9 @@ Message_encode (Message *This, Exception *ex, const CORBA_TCKindDesc *k,
                   Return = Message_encode(This,ex, &TCD_TypeCode, &TCD_null);
                if (Return) {
                   Return = Message_putULong (This, tc->kdesc->len);
-                  *((CORBA_ULong *) ulpnt) = (char *)This->currPnt - 
-                                      (char *) (ulpnt + sizeof(CORBA_ULong));
+                  *((CORBA_ULong *) (This->logicBuffer + ulpnt)) =
+                      (char *)This->currPnt - 
+                      (char *)(This->logicBuffer + ulpnt + sizeof(CORBA_ULong));
                }
                break;
          
